/*!
 * contentstack-webhook-listener
 * copyright (c) Contentstack LLC
 * MIT Licensed
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("debug");
const lodash_1 = require("lodash");
const core_1 = require("./core");
const defaults_1 = require("./defaults");
const logger_1 = require("./logger");
exports.setLogger = logger_1.setLogger;
const debug = debug_1.debug('webhook:listener');
let notify;
let config = {};
/**
 * Register a function that will get called when webhook is triggered.
 * @public
 * @param {function} consumer Function that will get called when webhook is triggered.
 */
function register(consumer) {
    if (typeof consumer !== 'function') {
        throw new Error('Provide function to notify consumer.');
    }
    debug('register called with %O', notify);
    notify = consumer;
    return true;
}
exports.register = register;
/**
 * Start webhook listener.
 * @public
 * @param {Object} userConfig JSON object that will override default config.
 * @param {Logger} customLogger Instance of a logger that should have info, debug, error, warn method.
 * @returns {Promise} Promise object represents http.Server
 */
function start(userConfig, customLogger) {
    if (customLogger) {
        logger_1.setLogger(customLogger);
    }
    return new Promise((resolve, reject) => {
        debug('start called with %O', userConfig);
        validateConfig(userConfig);
        // Override default with user config
        if (userConfig) {
            config = lodash_1.merge({}, defaults_1.defaultConfig, userConfig);
        }
        else {
            logger_1.logger.info('Starting listener with default configs');
            config = defaults_1.defaultConfig;
        }
        if (!notify) {
            logger_1.logger.error('Aborting start of webhook listener, since no function is provided to notify.');
            return reject(new Error(`Aborting start of webhook listener, since no function is provided to notify.`));
        }
        debug('starting with config: ' + JSON.stringify(config));
        const port = process.env.PORT || config.listener.port;
        const server = core_1.createListener(config, notify).listen(port, () => {
            logger_1.logger.info(`Server running at port ${port}`);
        });
        return resolve(server);
    });
}
exports.start = start;
/**
 * Get configuration.
 */
function getConfig() {
    return config;
}
exports.getConfig = getConfig;
/**
 * Validates configuration.
 * @param {object} customConfig JSON object that needs to validate.
 */
function validateConfig(customConfig) {
    if (customConfig && customConfig.listener) {
        if (customConfig.listener.endpoint) {
            if (typeof customConfig.listener.endpoint === 'string') {
                const reg = /^\//;
                if (!reg.test(customConfig.listener.endpoint)) {
                    customConfig.listener.endpoint = '/' + customConfig.listener.endpoint;
                }
            }
            else {
                throw new TypeError('Please provide valide listener.endpoint');
            }
        }
        if (customConfig.listener.port &&
            typeof customConfig.listener.port !== 'number') {
            throw new TypeError('Please provide valide listener.port');
        }
    }
}
//# sourceMappingURL=index.js.map